#include "chunk.h"

namespace harkio{
using namespace std;

/**********************************************************************
 　Chunk 基底クラスメンバ関数
 **********************************************************************/
int Chunk::getSize() {
	return size;
}

/**********************************************************************
 　FMT 派生クラスメンバ関数
 **********************************************************************/
bool FMT::isComplete() {
	return true;
}
void FMT::setSize() {
	Chunk::size = sizeof(Chunk::version.first) + sizeof(Chunk::version.second)
	+ sizeof(format_id) + sizeof(nMic) + sizeof(nFFT) + sizeof(nIR) + sizeof(nFS) + sizeof(MIC_TYPE)* nMic;
}
int FMT::getClassSize() {
	return getSize() + strlen(getIdentifier()) + sizeof(Chunk::size);
}

// ここから、フィールドデータ取得関数
FMT::FORMAT_ID FMT::getFormatId() {
	return format_id;
}
FMT::MIC_LIST_TYPE *FMT::getMic() {
	return &mic;
}
int FMT::getNMic() {
	return nMic;
}
int FMT::getNFFT() {
	return nFFT;
}
int FMT::getNIR() {
	return nIR;
}
int FMT::getNFS() {
	return nFS;
}
// マイク配列サイズ決定
FMT::MIC_LIST_TYPE *FMT::newMic() {
	mic.resize(0);
	mic.resize(nMic);
	return &mic;
}

// ファイル書き込み
void FMT::readFile(ifstream &fs) {
	format_id = M2PG; // ダミーデータ
	const char* identifier = getIdentifier();
	char cha_buf[strlen(identifier) + 1];

	// 識別子チェック
	memset(cha_buf, '\0', sizeof(cha_buf));
	fs.read((char*)cha_buf, strlen(identifier));
	if(fs.fail() || string(cha_buf) != identifier) {
		HARKIO_THROW_ERROR(string("FMT's identifier was't ") + identifier + ".");
	}

	// サイズ
	fs.read((char*)&size, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&size);
	// サイズの直後のシーク位置取得
	int start_seek = fs.tellg();

	// バージョン
	fs.read((char*)&version.first, sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.first);
	fs.read((char*)&version.second, sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.second);

	// フォーマットID
	fs.read((char*)&format_id, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&format_id);

	// マイク数
	fs.read((char*)&nMic, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&nMic);

	// NFFT
	fs.read((char*)&nFFT, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&nFFT);

	// IR
	fs.read((char*)&nIR, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&nIR);

	// NFS
	fs.read((char*)&nFS, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&nFS);

	// マイク座標配列作成
	newMic();
	for (int i=0; i < nMic; i++) {
		fs.read((char*)&mic[i].x, sizeof(BASE_TYPE));

		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all data.");
		}
		fs.read((char*)&mic[i].y, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all data.");
		}
		fs.read((char*)&mic[i].z, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all data.");
		}

		LToH(&mic[i].x);
		LToH(&mic[i].y);
		LToH(&mic[i].z);
	}
	// 読み取り開始時のシーク位置と、サイズを比較
	if ((int)fs.tellg() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellg() - start_seek << " Byte of " << identifier << " was read that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}
	return;
}
void FMT::writeFile(ofstream &fs) {
	const char* identifier = getIdentifier();

	// 識別子
	fs.write((char*)identifier, strlen(identifier));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}
	// サイズ
	fs.write((char*)LToHPtr(&size), sizeof(size));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// サイズの直後のシーク位置取得
	int start_seek = fs.tellp();

	// バージョン
	fs.write((char*)HToLPtr(&(version.first)), sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}
	fs.write((char*)HToLPtr(&(version.second)), sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// フォーマットID
	fs.write((char*)LToHPtr(&format_id), sizeof(format_id));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write FMT's data.");
	}
	// NMic
	fs.write((char*)LToHPtr(&nMic), sizeof(nMic));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write FMT's data.");
	}
	// NFFT
	fs.write((char*)LToHPtr(&nFFT), sizeof(nFFT));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write FMT's data.");
	}
	// IR
	fs.write((char*)LToHPtr(&nIR), sizeof(nIR));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write IR's data.");
	}
	// NFS
	fs.write((char*)LToHPtr(&nFS), sizeof(nFS));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write FMT's data.");
	}
	// Mic座標
	for(unsigned int i=0; i < mic.size(); i++) {
		fs.write((char*)LToHPtr(&(mic[i].x)), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write FMT's MIC data.");
		}
		fs.write((char*)LToHPtr(&(mic[i].y)), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write FMT's MIC data.");
		}
		fs.write((char*)LToHPtr(&(mic[i].z)), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write FMT's MIC data.");
		}
	}

	// 書き込み開始時のシーク位置と、サイズを比較
	if ((int)fs.tellp() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellp() - start_seek << " Byte of " << getIdentifier() << " was written that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}

	return;
}

/**********************************************************************
 　M2PG 派生クラスメンバ関数
 **********************************************************************/
bool M2PG::isComplete() {
	return true;
}
void M2PG::setSize() {
	Chunk::size =  sizeof(Chunk::version.first) + sizeof(Chunk::version.second)
		+ (sizeof(nd) + sizeof(nr))
		+ sizeof(MIC_TYPE) * nd * nr  // SRC
		+ sizeof(TF_TYPE) * nd * nr * fmt->getNMic() * (fmt->getNFFT()/2+1); // TF

}
int M2PG::getClassSize() {
	return getSize() + strlen(getIdentifier()) + sizeof(Chunk::size);
}

int M2PG::getND() {
	return nd;
}
int M2PG::getNR() {
	return nr;
}
M2PG::SRC_LIST_TYPE* M2PG::getSrc() {
	return &src;
}
M2PG::TF_LIST_TYPE *M2PG::getTF() {
	return &tf;
}

// ここから、フィールドデータ作成関数
// SRC
M2PG::SRC_LIST_TYPE *M2PG::newSrc() {
	src.resize(0); // 最初は初期化する
	src.resize(nd);
	for(int i = 0; i < nd; i++) {
		src[i].resize(nr);
	}
	return &src;
}
// TF
M2PG::TF_LIST_TYPE *M2PG::newTF() {
	tf.resize(0); //　最初は初期化する
	tf.resize(nd);
	for(int i = 0; i < nd; i++) {
		tf[i].resize(nr);
		for(int j = 0; j < nr; j++) {
			tf[i][j].resize(fmt->getNMic());
			for(int k = 0; k < fmt->getNMic(); k++) {
				tf[i][j][k].resize(fmt->getNFFT()/2+1);
			}
		}
	}
	return &tf;
}

void M2PG::readFile(ifstream &fs) {
	const char* identifier = getIdentifier();

	char cha_buf[strlen(identifier) + 1];

	// 識別子チェック
	memset(cha_buf, '\0', sizeof(cha_buf));
	fs.read((char*)&cha_buf, strlen(identifier));
	if(fs.fail() || string(cha_buf) != identifier) {
		HARKIO_THROW_ERROR(string("M2PG's identifier was't ") + identifier + ".");
	}

	// サイズ
	fs.read((char*)&size, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&size);
	// サイズの直後のシーク位置取得
	int start_seek = fs.tellg();

	// バージョン
	fs.read((char*)&version.first, sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.first);
	fs.read((char*)&version.second, sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.second);

	// nd
	fs.read((char*)&nd, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&nd);

	// nr
	fs.read((char*)&nr, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&nr);

	// 音源位置の座標配列の作成
	newSrc();
	for (int i=0; i < nd; i++) {
		for (int j=0; j < nr; j++) {
			fs.read((char*)&src[i][j].x, sizeof(BASE_TYPE));
			if(fs.fail()) {
				HARKIO_THROW_ERROR("Can't read all MIC data.");
			}
			fs.read((char*)&src[i][j].y, sizeof(BASE_TYPE));
			if(fs.fail()) {
				HARKIO_THROW_ERROR("Can't read all MIC data.");
			}
			fs.read((char*)&src[i][j].z, sizeof(BASE_TYPE));
			if(fs.fail()) {
				HARKIO_THROW_ERROR("Can't read all MIC data.");
			}
			LToH(&src[i][j].x);
			LToH(&src[i][j].y);
			LToH(&src[i][j].z);
		}
	}

	// 伝達関数配列作成
	newTF();
	for (int i=0; i < nd; i++) {
		for (int j=0; j < nr; j++) {
			for (int k=0; k < fmt->getNMic(); k++) {
				for (int l=0; l < fmt->getNFFT()/2+1; l++) {
					fs.read((char*)&tf[i][j][k][l].real(), sizeof(BASE_TYPE));
					if(fs.fail()) {
						HARKIO_THROW_ERROR("Can't read all TF data.");
					}
					fs.read((char*)&tf[i][j][k][l].imag(), sizeof(BASE_TYPE));
					if(fs.fail()) {
						HARKIO_THROW_ERROR("Can't read all TF data.");
					}
					LToH(&tf[i][j][k][l].real());
					LToH(&tf[i][j][k][l].imag());
				}
			}
		}
	}
	// 読み取り開始時のシーク位置と、サイズを比較
	if ((int)fs.tellg() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellg() - start_seek << " Byte of " << identifier << " was read that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}
	return;
}

// ファイル書き込み
void M2PG::writeFile(ofstream &fs) {
	const char* identifier = getIdentifier();

	// 識別子
	fs.write((char*)identifier, strlen(identifier));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write M2PG's data.");
	}
	// サイズ
	fs.write((char*)HToLPtr(&size), sizeof(size));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write M2PG's data.");
	}

	// サイズの直後のシーク位置取得
	int start_seek = fs.tellp();

	// バージョン
	fs.write((char*)HToLPtr(&(version.first)), sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}
	fs.write((char*)HToLPtr(&(version.second)), sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// ND
	fs.write((char*)HToLPtr(&nd), sizeof(nd));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write M2PG's data.");
	}
	// NR
	fs.write((char*)HToLPtr(&nr), sizeof(nr));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write M2PG's data.");
	}

	// Src座標
	for(int i=0; i < nd; i++) {
		for(int j=0; j < nr; j++) {
			fs.write((char*)HToLPtr(&(src[i][j]).x), sizeof(BASE_TYPE));
			if(fs.fail()) {
				HARKIO_THROW_ERROR("Can't write M2PG's Src data.");
			}
			fs.write((char*)HToLPtr(&(src[i][j]).y), sizeof(BASE_TYPE));
			if(fs.fail()) {
				HARKIO_THROW_ERROR("Can't write M2PG's Src data.");
			}
			fs.write((char*)HToLPtr(&(src[i][j]).z), sizeof(BASE_TYPE));
			if(fs.fail()) {
				HARKIO_THROW_ERROR("Can't write M2PG's Src data.");
			}
		}
	}
	// TFデータ
	for(int i=0; i < nd; i++) {
		for(int j=0; j < nr; j++) {
			for(int k=0; k < fmt->getNMic(); k++) {
				for(int l=0; l < fmt->getNFFT()/2+1; l++) {
					fs.write((char*)HToLPtr(&(tf[i][j][k][l]).real()), sizeof(BASE_TYPE));
					if(fs.fail()) {
						HARKIO_THROW_ERROR("Can't write M2PG's TF data.");
					}
					fs.write((char*)HToLPtr(&(tf[i][j][k][l]).imag()), sizeof(BASE_TYPE));
					if(fs.fail()) {
						HARKIO_THROW_ERROR("Can't write M2PG's TF data.");
					}
				}
			}
		}
	}

	// 書き込み開始時のシーク位置と、サイズを比較
	if ((int)fs.tellp() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellp() - start_seek << " Byte of " << getIdentifier() << " was written that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}

	return;
}

/**********************************************************************
 　 派生クラスメンバ関数
 **********************************************************************/
bool GTF::isComplete() {
	return true;
}
void GTF::setSize() {
	Chunk::size = sizeof(Chunk::version.first) + sizeof(Chunk::version.second)
		+ sizeof(n_src)
		+ sizeof(SRC_TYPE)* n_src // SRC
		+ sizeof(n_fi)
		+ sizeof(int) * n_fi
		+ sizeof(TF_TYPE) * n_src * fmt->getNMic() * n_fi; // TF

}
int GTF::getClassSize() {
	return getSize() + strlen(getIdentifier()) + sizeof(Chunk::size);
}

int GTF::getNSrc() {
	return n_src;
}
int GTF::getNFI() {
	return n_fi;
}
GTF::SRC_LIST_TYPE* GTF::getSrc() {
	return &src;
}
GTF::TF_LIST_TYPE *GTF::getTF() {
	return &tf;
}

vector<int> *GTF::getFI() {
	return &fi;
}

// ここから、フィールドデータ作成関数
// SRC
GTF::SRC_LIST_TYPE *GTF::newSrc() {
	src.resize(0); // 最初は初期化する
	src.resize(n_src);
	return &src;
}
// TF
GTF::TF_LIST_TYPE *GTF::newTF() {
	tf.resize(0); //　最初は初期化する
	tf.resize(n_src);
	for(int i = 0; i < n_src; i++) {
		tf[i].resize(fmt->getNMic());
		for(int j = 0; j < fmt->getNMic(); j++) {
			tf[i][j].resize(n_fi);
		}
	}
	return &tf;
}

// FI
vector<int> *GTF::newFI() {
	fi.resize(0); //　最初は初期化する
	fi.resize(n_fi);
	return &fi;
}

void GTF::readFile(ifstream &fs) {
	const char* identifier = getIdentifier();

	char cha_buf[strlen(identifier) + 1];

	// 識別子チェック
	memset(cha_buf, '\0', sizeof(cha_buf));
	fs.read((char*)&cha_buf, strlen(identifier));
	if(fs.fail() || string(cha_buf) != identifier) {
		HARKIO_THROW_ERROR(string("M2PG's identifier was't ") + identifier + ".");
	}

	// サイズ
	fs.read((char*)&size, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&size);
	// サイズの直後のシーク位置取得
	int start_seek = fs.tellg();

	// バージョン
	fs.read((char*)&version.first, sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.first);
	fs.read((char*)&version.second, sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.second);

	// NSrc
	fs.read((char*)&n_src, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&n_src);

	// 音源位置の座標配列の作成
	newSrc();
	for (int i=0; i < n_src; i++) {
		fs.read((char*)&src[i].x, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all MIC data.");
		}
		fs.read((char*)&src[i].y, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all MIC data.");
		}
		fs.read((char*)&src[i].z, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all MIC data.");
		}
		LToH(&src[i].x);
		LToH(&src[i].y);
		LToH(&src[i].z);
	}

	// n_fi
	fs.read((char*)&n_fi, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&n_fi);

	// 伝達関数の周波数インデックスから周波数へのマップ
	newFI();
	for (int i=0; i < n_fi; i++) {
		fs.read((char*)&fi[i], sizeof(int));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all data.");
		}
		LToH(&fi[i]);
	}

	// 伝達関数配列作成
	newTF();
	for (int i=0; i < n_src; i++) {
		for (int j=0; j < fmt->getNMic(); j++) {
			for (int k=0; k < n_fi; k++) {
				fs.read((char*)&tf[i][j][k].real(), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't read all TF data.");
				}
				fs.read((char*)&tf[i][j][k].imag(), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't read all TF data.");
				}
				LToH(&tf[i][j][k].real());
				LToH(&tf[i][j][k].imag());
			}
		}
	}
	// 読み取り開始時のシーク位置と、サイズを比較
	if ((int)fs.tellg() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellg() - start_seek << " Byte of " << identifier << " was read that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}
	return;
}

// ファイル書き込み
void GTF::writeFile(ofstream &fs) {
	const char* identifier = getIdentifier();

	// 識別子
	fs.write((char*)identifier, strlen(identifier));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write GTF's data.");
	}
	// サイズ
	fs.write((char*)HToLPtr(&size), sizeof(size));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write GTF's data.");
	}

	// サイズの直後のシーク位置取得
	int start_seek = fs.tellp();

	// バージョン
	fs.write((char*)HToLPtr(&(version.first)), sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}
	fs.write((char*)HToLPtr(&(version.second)), sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// NS
	fs.write((char*)HToLPtr(&n_src), sizeof(n_src));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write GTF's data.");
	}

	// Src座標
	for(int i=0; i < n_src; i++) {
		fs.write((char*)HToLPtr(&(src[i]).x), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write GTF's Src data.");
		}
		fs.write((char*)HToLPtr(&(src[i]).y), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write GTF's Src data.");
		}
		fs.write((char*)HToLPtr(&(src[i]).z), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write GTF's Src data.");
		}
	}

	// NFI
	fs.write((char*)HToLPtr(&n_fi), sizeof(n_fi));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write GTF's data.");
	}

	// 伝達関数のNFインデックスから周波数のマップ
	for(int i=0; i < n_fi; i++) {
		fs.write((char*)HToLPtr(&(fi[i])), sizeof(int));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write GTF's FI data.");
		}
	}

	// TFデータ
	for(int i=0; i < n_src; i++) {
		for(int j=0; j < fmt->getNMic(); j++) {
			for(int k=0; k < n_fi; k++) {
				fs.write((char*)HToLPtr(&(tf[i][j][k]).real()), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't write GTF's TF data.");
				}
				fs.write((char*)HToLPtr(&(tf[i][j][k]).imag()), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't write GTF's TF data.");
				}
			}
		}
	}

	// 書き込み開始時のシーク位置と、サイズを比較
	if ((int)fs.tellp() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellp() - start_seek << " Byte of " << getIdentifier() << " was written that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}
	return;
}

/**********************************************************************
 　SM 派生クラスメンバ関数
 **********************************************************************/
bool SM::isComplete() {
	return true;
}
void SM::setSize() {
	Chunk::size = sizeof(Chunk::version.first) + sizeof(Chunk::version.second)
		+ sizeof(n_src)
		+ sizeof(SRC_TYPE)* n_src // SRC
		+ sizeof(n_fi)
		+ sizeof(int) * n_fi
		+ sizeof(TF_TYPE) * n_src * fmt->getNMic() * n_fi; // TF

}
int SM::getClassSize() {
	return getSize() + strlen(getIdentifier()) + sizeof(Chunk::size);
}

int SM::getNSrc() {
	return n_src;
}
int SM::getNFI() {
	return n_fi;
}
SM::SRC_LIST_TYPE* SM::getSrc() {
	return &src;
}
SM::SM_LIST_TYPE *SM::getSM() {
	return &sm;
}

vector<int> *SM::getFI() {
	return &fi;
}

// ここから、フィールドデータ作成関数
// SRC
SM::SRC_LIST_TYPE *SM::newSrc() {
	src.resize(0); // 最初は初期化する
	src.resize(n_src);
	return &src;
}
// TF
SM::SM_LIST_TYPE *SM::newSM() {
	sm.resize(0); //　最初は初期化する
	sm.resize(n_src);
	for(int i = 0; i < n_src; i++) {
		sm[i].resize(fmt->getNMic());
		for(int j = 0; j < fmt->getNMic(); j++) {
			sm[i][j].resize(n_fi);
		}
	}
	return &sm;
}

// FI
vector<int> *SM::newFI() {
	fi.resize(0); //　最初は初期化する
	fi.resize(n_fi);
	return &fi;
}

void SM::readFile(ifstream &fs) {
	const char* identifier = getIdentifier();

	char cha_buf[strlen(identifier) + 1];

	// 識別子チェック
	memset(cha_buf, '\0', sizeof(cha_buf));
	fs.read((char*)&cha_buf, strlen(identifier));
	if(fs.fail() || string(cha_buf) != identifier) {
		HARKIO_THROW_ERROR(string("M2PG's identifier was't ") + identifier + ".");
	}

	// サイズ
	fs.read((char*)&size, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&size);
	// サイズの直後のシーク位置取得
	int start_seek = fs.tellg();

	// バージョン
	fs.read((char*)&version.first, sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.first);
	fs.read((char*)&version.second, sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.second);

	// NSrc
	fs.read((char*)&n_src, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&n_src);

	// 音源位置の座標配列の作成
	newSrc();
	for (int i=0; i < n_src; i++) {
		fs.read((char*)&src[i].x, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all MIC data.");
		}
		fs.read((char*)&src[i].y, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all MIC data.");
		}
		fs.read((char*)&src[i].z, sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all MIC data.");
		}
		LToH(&src[i].x);
		LToH(&src[i].y);
		LToH(&src[i].z);
	}

	// NFI
	fs.read((char*)&n_fi, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&n_fi);

	// 伝達関数の周波数インデックスから周波数へのマップ
	newFI();
	for (int i=0; i < n_fi; i++) {
		fs.read((char*)&fi[i], sizeof(int));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't read all data.");
		}
		LToH(&fi[i]);
	}

	// 分離行列配列作成
	newSM();
	for (int i=0; i < n_src; i++) {
		for (int j=0; j < fmt->getNMic(); j++) {
			for (int k=0; k < n_fi; k++) {
				fs.read((char*)&sm[i][j][k].real(), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't read all data.");
				}
				fs.read((char*)&sm[i][j][k].imag(), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't read all data.");
				}
				LToH(&sm[i][j][k].real());
				LToH(&sm[i][j][k].imag());
			}
		}
	}
	// 読み取り開始時のシーク位置と、サイズを比較
	if ((int)fs.tellg() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellg() - start_seek << " Byte of " << identifier << " was read that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}
	return;
}

// ファイル書き込み
void SM::writeFile(ofstream &fs) {
	const char* identifier = getIdentifier();

	// 識別子
	fs.write((char*)identifier, strlen(identifier));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}
	// サイズ
	fs.write((char*)HToLPtr(&size), sizeof(size));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// サイズの直後のシーク位置取得
	int start_seek = fs.tellp();

	// バージョン
	fs.write((char*)HToLPtr(&(version.first)), sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}
	fs.write((char*)HToLPtr(&(version.second)), sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// NS
	fs.write((char*)HToLPtr(&n_src), sizeof(n_src));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// Src座標
	for(int i=0; i < n_src; i++) {
		fs.write((char*)HToLPtr(&(src[i]).x), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write all data.");
		}
		fs.write((char*)HToLPtr(&(src[i]).y), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write all data.");
		}
		fs.write((char*)HToLPtr(&(src[i]).z), sizeof(BASE_TYPE));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write all data.");
		}
	}

	// NFI
	fs.write((char*)HToLPtr(&n_fi), sizeof(n_fi));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// 分離行列のNFインデックスから周波数のマップ
	for(int i=0; i < n_fi; i++) {
		fs.write((char*)HToLPtr(&(fi[i])), sizeof(int));
		if(fs.fail()) {
			HARKIO_THROW_ERROR("Can't write all FI data.");
		}
	}

	// SMデータ
	for(int i=0; i < n_src; i++) {
		for(int j=0; j < fmt->getNMic(); j++) {
			for(int k=0; k < n_fi; k++) {
				fs.write((char*)HToLPtr(&(sm[i][j][k]).real()), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't write all data.");
				}
				fs.write((char*)HToLPtr(&(sm[i][j][k]).imag()), sizeof(BASE_TYPE));
				if(fs.fail()) {
					HARKIO_THROW_ERROR("Can't write all data.");
				}
			}
		}
	}

	// 書き込み開始時のシーク位置と、サイズを比較
	if ((int)fs.tellp() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellp() - start_seek << " Byte of " << getIdentifier() << " was written that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}
	return;
}

/**********************************************************************
 　SPEC 派生クラスメンバ関数
 **********************************************************************/
bool SPEC::isComplete() {
	return true;
}

void SPEC::setSize() {
	Chunk::size =  sizeof(Chunk::version.first) + sizeof(Chunk::version.second)
		+ sizeof(date) + sizeof(location)
		+ sizeof(robot) + sizeof(array_type)
		+ sizeof(creator) + sizeof(memo);
}

int SPEC::getClassSize() {
	return getSize() + strlen(getIdentifier()) + sizeof(Chunk::size);
}

void SPEC::readFile(ifstream &fs) {
	const char* identifier = getIdentifier();

	char cha_buf[strlen(identifier) + 1];

	// 識別子チェック
	memset(cha_buf, '\0', sizeof(cha_buf));
	fs.read((char*)&cha_buf, strlen(identifier));
	if(fs.fail() || string(cha_buf) != identifier) {
		HARKIO_THROW_ERROR(string("M2PG's identifier was't ") + identifier + ".");
	}

	// サイズ
	fs.read((char*)&size, sizeof(int));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(&size);
	// サイズの直後のシーク位置取得
	int start_seek = fs.tellg();

	// バージョン
	fs.read((char*)&version.first, sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.first);
	fs.read((char*)&version.second, sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	LToH(version.second);

	// date
	fs.read((char*)date, sizeof(date));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	// location
	fs.read((char*)location, sizeof(location));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	// robot
	fs.read((char*)robot, sizeof(robot));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	// array_type
	fs.read((char*)array_type, sizeof(array_type));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	// creator
	fs.read((char*)creator, sizeof(creator));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}
	// memo
	fs.read((char*)memo, sizeof(memo));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't read all data.");
	}

	// 読み取り開始時のシーク位置と、サイズを比較
	if ((int)fs.tellg() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellg() - start_seek << " Byte of " << identifier << " was read that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}
	return;
}

// ファイル書き込み
void SPEC::writeFile(ofstream &fs) {
	const char* identifier = getIdentifier();

	// 識別子
	fs.write((char*)identifier, strlen(identifier));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}
	// サイズ
	fs.write((char*)HToLPtr(&size), sizeof(size));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}

	// サイズの直後のシーク位置取得
	int start_seek = fs.tellp();

	// バージョン
	fs.write((char*)HToLPtr(&(version.first)), sizeof(version.first));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}
	fs.write((char*)HToLPtr(&(version.second)), sizeof(version.second));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write all data.");
	}

	// date
	fs.write((char*)date, sizeof(date));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}
	// location
	fs.write((char*)location, sizeof(location));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}
	// robot
	fs.write((char*)robot, sizeof(robot));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}
	// array_type
	fs.write((char*)array_type, sizeof(array_type));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}
	// creator
	fs.write((char*)creator, sizeof(creator));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}
	// memo
	fs.write((char*)memo, sizeof(memo));
	if(fs.fail()) {
		HARKIO_THROW_ERROR("Can't write SPEC's data.");
	}

	// 書き込み開始時のシーク位置と、サイズを比較
	if ((int)fs.tellp() - start_seek != getSize()) {
		stringstream ss;
		ss << (int)fs.tellp() - start_seek << " Byte of " << getIdentifier() << " was written that header size is " << getSize() << " Byte.";
		HARKIO_THROW_ERROR(ss.str());
	}

	return;
}

} // namespace
